# See article: http://fiber-space.de/wordpress/?p=1579
# for a more detailed description.

__all__ = ["fuzzy_compare"]

def is_allowed(i,j,k,N1,N2,visited):
    m = visited.get((i,j),-1)
    if k>m:
        visited[(i,j)] = k
        r1 = N1-i
        r2 = N2-j
        d = max(r2-r1, r1-r2)
        if d<=k:
            return True
    return False

def fuzzy_compare(S1, S2, k):
    '''
    Compares two strings with a bound number of errors.

    Returns True, if edit_distance(S1, S2)<=k,
            False otherwise.
    '''
    N1 = len(S1)
    N2 = len(S2)
    visited = {}
    i = j = 0
    stack = []
    while True:
        if N1-i<=k and N2-j<=k:
            return True
        try:
            if S1[i] == S2[j]:
                visited[(i,j)] = k
                i+=1
                j+=1
                continue
        except IndexError:
            if stack:
                i, j, k = stack.pop()
            else:
                return False
        if k == 0:
            if stack:
                i, j, k = stack.pop()
            else:
                return False
        else:
            if is_allowed(i+1, j+1, k-1, N1, N2, visited):
                stack.append((i,j,k))
                i, j, k = i+1, j+1, k-1
            elif is_allowed(i+1, j, k-1, N1, N2, visited):
                stack.append((i,j,k))
                i, j, k = i+1, j, k-1
            elif is_allowed(i, j+1, k-1, N1, N2, visited):
                i, j, k = i, j+1, k-1
            elif stack:
                i, j, k = stack.pop()
            else:
                return False



