import string

# For k = 1 and small alphabets the get_string_variants function is the recommended
# way to apply approximate string matches -- no PrefixTree is needed

def get_string_variants(S, alphabet = string.letters):
    '''
    For a given string S produce all Strings S' over alphabet with edit-distance(S,S')<=1.

    The default alphabet is given by string.letters.
    '''
    S = S.lower()
    variants = set()
    for i in range(len(S)+1):
        variants.add(S[:i]+S[i+1:])
        for s in alphabet:
            variants.add(S[:i]+s+S[i:])   # insert at i
            variants.add(S[:i]+s+S[i+1:]) # subst i
    return variants


# generalization of get_string_variants
#
# This is function is not meant to be an efficient implementation and is used for testing purposes
# only.

def get_higher_string_variants(S, k, alphabet = string.letters):
    '''
    For a given string S produce all Strings S' over alphabet with edit-distance(S,S')<=k.

    The default alphabet is given by string.letters.
    '''
    variants = get_string_variants(S)
    if k == 1:
        return variants
    else:
        unified = set()
        for s in variants:
            unified.update(get_higher_string_variants(s, k-1, alphabet))
        return unified


